--[[
Name: StatsPane_StatMods
Description: Displays stat modification values in the character stat pane.
Revision: $Revision: 2 $
Author: Whitetooth
Email: hotdogee [at] gmail [dot] com
LastUpdate: $Date: 2010-10-19 17:59:44 +0000 (Tue, 19 Oct 2010) $
]]

local addonName = "StatsPane_StatMods"
---------------
-- Libraries --
---------------
local StatLogic = LibStub("LibStatLogic-1.2")
local StatsPane = LibStub("LibStatsPane-1.0")

-- Locale --
local L
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "enUS", true)
L["Stat Modifiers"] = true
L["Hide inactive stat modifiers"] = true
L[" "] = true -- some locales don't use spaces between words, like Chinese
L["<"] = true
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "zhTW")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "zhCN")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "koKR")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "deDE")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "frFR")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "ruRU")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):NewLocale(addonName, "esES")
if L then
-- L["Enables RatingBuster to calculate selected buff effects even if you don't really have them"] = ""
-- L["$class Self Buffs"] = ""
-- L["Raid Buffs"] = ""
end
L = LibStub("AceLocale-3.0"):GetLocale(addonName)
--------------------
-- AceAddon Setup --
--------------------
-- AceAddon Initialization
local StatMods = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0")


local iconPlus = "|TInterface\\Icons\\Spell_ChargePositive:0|t"
local iconMultiply = "|TInterface\\AddOns\\StatsPane_StatMods\\Multiply:0|t"
local iconSchools = {
  "|TInterface\\Icons\\INV_Weapon_ShortBlade_05:0|t",
  "|TInterface\\Icons\\INV_Weapon_Bow_07:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon2:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon3:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon4:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon5:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon6:0|t",
  "|TInterface\\PaperDollInfoFrame\\SpellSchoolIcon7:0|t",
}

-- Generate stat labels
local statList = {}
for k, v in pairs(StatLogic.StatModInfo) do
  -- Parse StatModID
  -- 1. ADD_AP_MOD_INT
  -- 2. ADD_CRIT_TAKEN
  -- 3. MOD_STA
  if strfind(k, "^ADD_([A-Z_]-)_MOD_([A-Z_]-)$") then
    local _, _, s1, s2 = strfind(k, "^ADD_([A-Z_]-)_MOD_([A-Z_]-)$")
    local d1, d2 -- display
    if strfind(s1, "^PET_([A-Z_]-)$") then
      _, _, d1 = strfind(s1, "^PET_([A-Z_]-)$")
      d1 = PET..L[" "]..select(2, StatLogic:GetStatNameFromID(d1))
    else
      d1 = select(2, StatLogic:GetStatNameFromID(s1))
    end
    if strfind(s2, "^PET_([A-Z_]-)$") then
      _, _, d2 = strfind(s2, "^PET_([A-Z_]-)$")
      d2 = PET..L[" "]..select(2, StatLogic:GetStatNameFromID(d2))
    else
      d2 = select(2, StatLogic:GetStatNameFromID(s2))
    end
    if not d1 then error("Unable to parse: "..s1) end
    if not d2 then error("Unable to parse: "..s2) end
    statList[k] = iconPlus..L[" "]..d1..L["<"]..iconMultiply..L[" "]..d2
  elseif strfind(k, "^ADD_([A-Z_]-)$") then
    local _, _, s1 = strfind(k, "^ADD_([A-Z_]-)$")
    statList[k] = iconPlus..L[" "]..select(2, StatLogic:GetStatNameFromID(s1))
  elseif strfind(k, "^MOD_([A-Z_]-)$") then
    local _, _, s1 = strfind(k, "^MOD_([A-Z_]-)$")
    if not StatLogic:GetStatNameFromID(s1) then error("Unable to parse: "..s1) end
    statList[k] = iconMultiply..L[" "]..select(2, StatLogic:GetStatNameFromID(s1))
  else
    error("Unable to parse: "..k)
  end
end

local function pairsByKeys(t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
  table.sort(a, f)
  local i = 0      -- iterator variable
  local iter = function ()   -- iterator function
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end
  return iter
end

local function SetStatMod(statFrame, modId, schoolId, schoolIndex)
  local value
  if schoolIndex then
    value = StatLogic:GetStatMod(modId, schoolId)
  else
    value = StatLogic:GetStatMod(modId)
  end
  local base = StatLogic.StatModInfo[modId].initialValue + StatLogic.StatModInfo[modId].finalAdjust
  value = (value - base) * 100
  if value == 0 then
    statFrame:Hide()
    return
  end
  local label = statList[modId]
  if schoolIndex then
    label = label.." "..iconSchools[schoolIndex]
  end
  PaperDollFrame_SetLabelAndText(statFrame, label, value, true)
  statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..label.." "..value..FONT_COLOR_CODE_CLOSE
  statFrame.tooltip2 = modId
  statFrame:Show()
end

local pane = {
  Category = L["Stat Modifiers"], -- string
  StatInfo = {
  },
}

for k, v in pairsByKeys(statList) do
  if StatLogic.StatModInfo[k].school then
    for sk, sv in ipairs(StatLogic.SpellSchools) do
      local statTable = {
        id = addonName..k..sv,
        updateFunc = function(statFrame, unit)
          SetStatMod(statFrame, k, sv, sk)
        end,
      }
      tinsert(pane.StatInfo, statTable)
    end
  else
    local statTable = {
      id = addonName..k,
      updateFunc = function(statFrame, unit)
        SetStatMod(statFrame, k)
      end,
    }
    tinsert(pane.StatInfo, statTable)
  end
end

StatsPane:AddPane(pane)